
from gtoc import GTOC
import subprocess
import random

import argparse

projBin="/tcenas/home/roche/projects/resp_t4/proj_app/bin/proj"


parser = argparse.ArgumentParser(description='Run compare tests between Python and PROJ implementations of GTOC projection.')
parser.add_argument('-n', '--numtests', type=int, default=100, help='Specify the number of tests to execute')
parser.add_argument('-e', '--eps', type=float, default=1e-10, help='Specify the expected relative precision epsilon value to use')

args = parser.parse_args()
# print(args.accumulate(args.integers))

# print("Executing GTOC test here!")

numTests = args.numtests
print("Executing %d tests..." % numTests)
eps = args.eps

for i in range(numTests):
    # lon = 12.5
    # lat = 42.0

    lat = random.uniform(-88.0, 88.0)
    lon = random.uniform(-178.0, 178.0)
    # lon = random.uniform(-88.0, 88.0)
    lon0 = random.uniform(-178.0, 178.0)
    lat0 = random.uniform(-88.0, 88.0)
    az0 = random.uniform(-178.0, 178.0)
    theta0 = random.uniform(-88.0, 88.0)
    
    sz0 = None
    # Note: we also want to test the auto compute of the scale, so we might either provide a value here or jsut send a None:
    if random.random()>0.5:
        # Should represent a realistic earth radius value:
        sz0 = random.uniform(6350000.0, 6390000.0)

    # Create a new instance:
    proj = GTOC(lon_0=lon0, lat_0=lat0, azRot=az0, thetaEq=theta0, scale=sz0)
    # proj = GTOC(lon_0=0.0, lat_0=0.0, azRot=34.5, thetaEq=45.6)
    # proj = GTOC(lon_0=12.3, lat_0=23.4, azRot=34.5, thetaEq=45.6)

    # print("Converting coords: (12.5, 42.0)")
    x,y = proj.Earth2Map(lon, lat)

    # print("Got map coords: (%.14f, %.14f)" % (x,y))
    lon2,lat2 = proj.Map2Earth(x, y)

    sz0Val = sz0 if sz0 is not None else 0.0

    if abs(lat-lat2) > eps or abs(lon-lon2) > eps:
        raise Exception("Invalid Python GTOC convertion: (%.12f,%.12f) != (%.12f,%.12f) with params (%.6f, %.6f, %.6f, %.6f, %.6f)" %
                        (lat, lon, lat2, lon2, lon0, lat0, az0, theta0, sz0Val))

    # print("Converted coords back: (%.14f, %.14f)" % (lat,lon))

    # Try running one instance of our own proj binary:
    # result = subprocess.run([projBin, '--version'], stdout=subprocess.PIPE) 

    # Note: if sz0 is None, then we should not send any scale parameter at all below:
    cmd = [projBin, '-d', '12', '+proj=gtoc', '+lon_0=%.12f' % lon0, '+lat_0=%.12f' % lat0, '+az_0=%.12f' % az0, '+theta_0=%.12f' % theta0]
    if sz0 is not None:
        cmd.append('+scale=%.12f' % sz0) 

    result = subprocess.run(cmd, stdout=subprocess.PIPE, input='%.12f %.12f' % (lon, lat), encoding='ascii')
    res = result.stdout.split()
    xx = float(res[0])
    yy = float(res[1])

    # print("Got: %.12f %.12f" % (xx, yy))
    # print("Expected: %.12f %.12f" % (x, y))

    if abs(x-xx)/x > eps or abs(y-yy)/y > eps:
        raise Exception("Found mismatch: (%.12f,%.12f) != (%.12f,%.12f) for coords (%.12f, %.12f) with params (%.6f, %.6f, %.6f, %.6f, %.6f)" %
                        (x, y, xx, yy, lat, lon, lon0, lat0, az0, theta0, sz0Val))

    # Now we convert back the map coordinates to lat/lon coordinates using the C++ implementation:
    cmd.insert(1, "-I")

    result = subprocess.run(cmd, stdout=subprocess.PIPE, input='%.12f %.12f' % (xx, yy), encoding='ascii')
    res = result.stdout.split()
    lon2 = float(res[0])
    lat2 = float(res[1])

    if abs(lat-lat2)/lat > eps or abs(lon-lon2)/lon > eps:
        raise Exception("Found inverse mismatch: (%.12f,%.12f) != (%.12f,%.12f) for coords (%.12f, %.12f) with params (%.6f, %.6f, %.6f, %.6f, %.6f)" %
                        (x, y, xx, yy, lat, lon, lon0, lat0, az0, theta0, sz0Val))

    print("(%.6f, %.6f, %.6f, %.6f, %.6f, %.6f, %.6f): OK" % (lat, lon, lon0, lat0, az0, theta0, sz0Val))

print("Done.")